@@ -1,5 +1,3 @@
-# python >= 3.6
-
 class Foo:
     bar = 1
 
@@ -25,3 +23,14 @@ Fr'sasdf'
 
 #? 7 str()
 Fr'''sasdf''' + ''
+
+#? ['upper']
+f'xyz'.uppe
+
+
+#? 3 []
+f'f'
+
+# Github #1248
+#? int()
+{"foo": 1}[f"foo"]
