@@ -1,5 +1,5 @@
 def from_names():
-    #? ['mod1']
+    #? ['mod1', 'base']
     from import_tree.pkg.
     #? ['path']
     from os.
@@ -12,6 +12,8 @@ def from_names_goto():
 def builtin_test():
     #? ['math']
     import math
+    #? ['mmap']
+    import mmap
 
 # -----------------
 # completions within imports
@@ -21,11 +23,9 @@ def builtin_test():
 import sqlite3
 
 # classes is a local module that has an __init__.py and can therefore not be
-# found. test can be found.
+# found.
 #? []
 import classes
-#? ['test']
-import test
 
 #? ['timedelta']
 from datetime import timedel
@@ -73,9 +73,9 @@ from import_tree.pkg import pkg
 from import_tree.pkg.mod1 import not_existant,    # whitespace before
 #? ['a', 'foobar', '__name__', '__doc__', '__file__', '__package__']
 from import_tree.pkg.mod1 import not_existant, 
-#? 22 ['mod1']
+#? 22 ['mod1', 'base']
 from import_tree.pkg. import mod1
-#? 17 ['mod1', 'mod2', 'random', 'pkg', 'rename1', 'rename2', 'classes', 'globals', 'recurse_class1', 'recurse_class2', 'invisible_pkg', 'flow_import']
+#? 17 ['mod1', 'mod2', 'random', 'pkg', 'references', 'rename1', 'rename2', 'classes', 'globals', 'recurse_class1', 'recurse_class2', 'invisible_pkg', 'flow_import']
 from import_tree. import pkg
 
 #? 18 ['pkg']
