@@ -1,5 +1,5 @@
 import os
-import re
+import sys
 import subprocess
 
 import pytest
@@ -7,10 +7,16 @@ import pytest
 from . import helpers
 from . import run
 from . import refactor
-
-import jedi
-from jedi.api.environment import InterpreterEnvironment
+from jedi import InterpreterEnvironment, get_system_environment
 from jedi.inference.compiled.value import create_from_access_path
+from jedi.api.interpreter import MixedModuleContext
+
+# For interpreter tests sometimes the path of this directory is in the sys
+# path, which we definitely don't want. So just remove it globally.
+try:
+    sys.path.remove(helpers.test_dir)
+except ValueError:
+    pass
 
 
 def pytest_addoption(parser):
@@ -66,9 +72,11 @@ def pytest_generate_tests(metafunc):
 
     if 'refactor_case' in metafunc.fixturenames:
         base_dir = metafunc.config.option.refactor_case_dir
+        cases = list(refactor.collect_dir_tests(base_dir, test_files))
         metafunc.parametrize(
-            'refactor_case',
-            refactor.collect_dir_tests(base_dir, test_files))
+            'refactor_case', cases,
+            ids=[c.refactor_type + '-' + c.name for c in cases]
+        )
 
     if 'static_analysis_case' in metafunc.fixturenames:
         base_dir = os.path.join(os.path.dirname(__file__), 'static_analysis')
@@ -90,11 +98,15 @@ def collect_static_analysis_tests(base_dir, test_files):
 
 @pytest.fixture(scope='session')
 def venv_path(tmpdir_factory, environment):
-    if environment.version_info.major < 3:
-        pytest.skip("python -m venv does not exist in Python 2")
+    if isinstance(environment, InterpreterEnvironment):
+        # The environment can be a tox virtualenv environment which we don't
+        # want, so use the system environment.
+        environment = get_system_environment(
+            '.'.join(str(x) for x in environment.version_info[:2])
+        )
 
     tmpdir = tmpdir_factory.mktemp('venv_path')
-    dirname = os.path.join(tmpdir.dirname, 'venv')
+    dirname = os.path.join(tmpdir.strpath, 'venv')
 
     # We cannot use the Python from tox because tox creates virtualenvs and
     # they have different site.py files that work differently than the default
@@ -112,7 +124,8 @@ def venv_path(tmpdir_factory, environment):
         executable_name = os.path.basename(environment.executable)
         executable_path = os.path.join(prefix, 'bin', executable_name)
 
-    subprocess.call([executable_path, '-m', 'venv', dirname])
+    return_code = subprocess.call([executable_path, '-m', 'venv', dirname])
+    assert return_code == 0, return_code
     return dirname
 
 
@@ -144,3 +157,14 @@ def create_compiled_object(inference_state):
         inference_state,
         inference_state.compiled_subprocess.create_simple_object(obj)
     )
+
+
+@pytest.fixture(params=[False, True])
+def class_findable(monkeypatch, request):
+    if not request.param:
+        monkeypatch.setattr(
+            MixedModuleContext,
+            '_get_mixed_object',
+            lambda self, compiled_object: compiled_object.as_context()
+        )
+    return request.param
