@@ -3,21 +3,14 @@ Tests of ``jedi.api.Interpreter``.
 """
 import sys
 import warnings
+import typing
 
 import pytest
 
 import jedi
-from jedi._compatibility import is_py3, py_version
-from jedi.inference.compiled import mixed, value
+from jedi.inference.compiled import mixed
 from importlib import import_module
 
-if py_version > 30:
-    def exec_(source, global_map):
-        exec(source, global_map)
-else:
-    eval(compile("""def exec_(source, global_map):
-                        exec source in global_map """, 'blub', 'exec'))
-
 
 class _GlobalNameSpace:
     class SideEffectContainer:
@@ -26,7 +19,7 @@ class _GlobalNameSpace:
 
 def get_completion(source, namespace):
     i = jedi.Interpreter(source, [namespace])
-    completions = i.completions()
+    completions = i.complete()
     assert len(completions) == 1
     return completions[0]
 
@@ -78,8 +71,8 @@ def test_numpy_like_non_zero():
 
     # just assert these do not raise. They (strangely) trigger different
     # codepath
-    get_completion('d["a"].some', {'d':d})
-    get_completion('x.some', {'x':x})
+    get_completion('d["a"].some', {'d': d})
+    get_completion('x.some', {'x': x})
 
 
 def test_nested_resolve():
@@ -111,63 +104,43 @@ def test_side_effect_completion():
 def _assert_interpreter_complete(source, namespace, completions,
                                  **kwds):
     script = jedi.Interpreter(source, [namespace], **kwds)
-    cs = script.completions()
+    cs = script.complete()
     actual = [c.name for c in cs]
     assert sorted(actual) == sorted(completions)
 
 
 def test_complete_raw_function():
     from os.path import join
-    _assert_interpreter_complete('join("").up',
-                                 locals(),
-                                 ['upper'])
+    _assert_interpreter_complete('join("").up', locals(), ['upper'])
 
 
 def test_complete_raw_function_different_name():
     from os.path import join as pjoin
-    _assert_interpreter_complete('pjoin("").up',
-                                 locals(),
-                                 ['upper'])
+    _assert_interpreter_complete('pjoin("").up', locals(), ['upper'])
 
 
 def test_complete_raw_module():
     import os
-    _assert_interpreter_complete('os.path.join("a").up',
-                                 locals(),
-                                 ['upper'])
+    _assert_interpreter_complete('os.path.join("a").up', locals(), ['upper'])
 
 
 def test_complete_raw_instance():
     import datetime
     dt = datetime.datetime(2013, 1, 1)
-    completions = ['time', 'timetz', 'timetuple']
-    if is_py3:
-        completions += ['timestamp']
-    _assert_interpreter_complete('(dt - dt).ti',
-                                 locals(),
-                                 completions)
+    completions = ['time', 'timetz', 'timetuple', 'timestamp']
+    _assert_interpreter_complete('(dt - dt).ti', locals(), completions)
 
 
 def test_list():
     array = ['haha', 1]
-    _assert_interpreter_complete('array[0].uppe',
-                                 locals(),
-                                 ['upper'])
-    _assert_interpreter_complete('array[0].real',
-                                 locals(),
-                                 [])
+    _assert_interpreter_complete('array[0].uppe', locals(), ['upper'])
+    _assert_interpreter_complete('array[0].real', locals(), [])
 
     # something different, no index given, still just return the right
-    _assert_interpreter_complete('array[int].real',
-                                 locals(),
-                                 ['real'])
-    _assert_interpreter_complete('array[int()].real',
-                                 locals(),
-                                 ['real'])
+    _assert_interpreter_complete('array[int].real', locals(), ['real'])
+    _assert_interpreter_complete('array[int()].real', locals(), ['real'])
     # inexistent index
-    _assert_interpreter_complete('array[2].upper',
-                                 locals(),
-                                 ['upper'])
+    _assert_interpreter_complete('array[2].upper', locals(), ['upper'])
 
 
 def test_getattr():
@@ -181,9 +154,7 @@ def test_slice():
     class Foo1:
         bar = []
     baz = 'xbarx'
-    _assert_interpreter_complete('getattr(Foo1, baz[1:-1]).append',
-                                 locals(),
-                                 ['append'])
+    _assert_interpreter_complete('getattr(Foo1, baz[1:-1]).append', locals(), ['append'])
 
 
 def test_getitem_side_effects():
@@ -213,7 +184,6 @@ def test_property_warnings(stacklevel, allow_unsafe_getattr):
     _assert_interpreter_complete('foo.prop.uppe', locals(), expected)
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Ignore Python 2, because EOL")
 @pytest.mark.parametrize('class_is_findable', [False, True])
 def test__getattr__completions(allow_unsafe_getattr, class_is_findable):
     class CompleteGetattr(object):
@@ -255,6 +225,7 @@ def allow_unsafe_getattr(request, monkeypatch):
 
 def test_property_error_oldstyle(allow_unsafe_getattr):
     lst = []
+
     class Foo3:
         @property
         def bar(self):
@@ -266,7 +237,7 @@ def test_property_error_oldstyle(allow_unsafe_getattr):
     _assert_interpreter_complete('foo.bar.baz', locals(), [])
 
     if allow_unsafe_getattr:
-        assert lst == [1, 1]
+        assert lst == [1]
     else:
         # There should not be side effects
         assert lst == []
@@ -274,6 +245,7 @@ def test_property_error_oldstyle(allow_unsafe_getattr):
 
 def test_property_error_newstyle(allow_unsafe_getattr):
     lst = []
+
     class Foo3(object):
         @property
         def bar(self):
@@ -285,7 +257,7 @@ def test_property_error_newstyle(allow_unsafe_getattr):
     _assert_interpreter_complete('foo.bar.baz', locals(), [])
 
     if allow_unsafe_getattr:
-        assert lst == [1, 1]
+        assert lst == [1]
     else:
         # There should not be side effects
         assert lst == []
@@ -298,19 +270,18 @@ def test_property_content():
             return 1
 
     foo = Foo3()
-    def_, = jedi.Interpreter('foo.bar', [locals()]).goto_definitions()
+    def_, = jedi.Interpreter('foo.bar', [locals()]).infer()
     assert def_.name == 'int'
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Ignore Python 2, because EOL")
 def test_param_completion():
     def foo(bar):
         pass
 
     lambd = lambda xyz: 3
 
-    _assert_interpreter_complete('foo(bar', locals(), ['bar'])
-    assert bool(jedi.Interpreter('lambd(xyz', [locals()]).completions()) == is_py3
+    _assert_interpreter_complete('foo(bar', locals(), ['bar='])
+    _assert_interpreter_complete('lambd(xyz', locals(), ['xyz='])
 
 
 def test_endless_yield():
@@ -320,50 +291,51 @@ def test_endless_yield():
     _assert_interpreter_complete('list(lst)[9000].rea', locals(), ['real'])
 
 
-@pytest.mark.skipif('py_version < 33', reason='inspect.signature was created in 3.3.')
 def test_completion_params():
     foo = lambda a, b=3: None
 
     script = jedi.Interpreter('foo', [locals()])
-    c, = script.completions()
-    assert [p.name for p in c.params] == ['a', 'b']
-    assert c.params[0].infer() == []
-    t, = c.params[1].infer()
+    c, = script.complete()
+    sig, = c.get_signatures()
+    assert [p.name for p in sig.params] == ['a', 'b']
+    assert sig.params[0].infer() == []
+    t, = sig.params[1].infer()
     assert t.name == 'int'
 
 
-@pytest.mark.skipif('py_version < 33', reason='inspect.signature was created in 3.3.')
 def test_completion_param_annotations():
-    # Need to define this function not directly in Python. Otherwise Jedi is to
+    # Need to define this function not directly in Python. Otherwise Jedi is too
     # clever and uses the Python code instead of the signature object.
     code = 'def foo(a: 1, b: str, c: int = 1.0) -> bytes: pass'
-    exec_(code, locals())
+    exec(code, locals())
     script = jedi.Interpreter('foo', [locals()])
-    c, = script.completions()
-    a, b, c = c.params
+    c, = script.complete()
+    sig, = c.get_signatures()
+    a, b, c = sig.params
     assert a.infer() == []
     assert [d.name for d in b.infer()] == ['str']
     assert {d.name for d in c.infer()} == {'int', 'float'}
 
-    d, = jedi.Interpreter('foo()', [locals()]).goto_definitions()
+    assert a.description == 'param a: 1'
+    assert b.description == 'param b: str'
+    assert c.description == 'param c: int=1.0'
+
+    d, = jedi.Interpreter('foo()', [locals()]).infer()
     assert d.name == 'bytes'
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Ignore Python 2, because EOL")
 def test_keyword_argument():
     def f(some_keyword_argument):
         pass
 
-    c, = jedi.Interpreter("f(some_keyw", [{'f': f}]).completions()
-    assert c.name == 'some_keyword_argument'
+    c, = jedi.Interpreter("f(some_keyw", [{'f': f}]).complete()
+    assert c.name == 'some_keyword_argument='
     assert c.complete == 'ord_argument='
 
-    # This needs inspect.signature to work.
-    if is_py3:
-        # Make it impossible for jedi to find the source of the function.
-        f.__name__ = 'xSOMETHING'
-        c, = jedi.Interpreter("x(some_keyw", [{'x': f}]).completions()
-        assert c.name == 'some_keyword_argument'
+    # Make it impossible for jedi to find the source of the function.
+    f.__name__ = 'xSOMETHING'
+    c, = jedi.Interpreter("x(some_keyw", [{'x': f}]).complete()
+    assert c.name == 'some_keyword_argument='
 
 
 def test_more_complex_instances():
@@ -375,13 +347,13 @@ def test_more_complex_instances():
         def wow(self):
             return Something()
 
-    #script = jedi.Interpreter('Base().wow().foo', [locals()])
-    #c, = script.completions()
-    #assert c.name == 'foo'
+    script = jedi.Interpreter('Base().wow().foo', [locals()])
+    c, = script.complete()
+    assert c.name == 'foo'
 
     x = Base()
     script = jedi.Interpreter('x.wow().foo', [locals()])
-    c, = script.completions()
+    c, = script.complete()
     assert c.name == 'foo'
 
 
@@ -397,7 +369,7 @@ def test_repr_execution_issue():
     er = ErrorRepr()
 
     script = jedi.Interpreter('er', [locals()])
-    d, = script.goto_definitions()
+    d, = script.infer()
     assert d.name == 'ErrorRepr'
     assert d.type == 'instance'
 
@@ -412,16 +384,12 @@ def test_dir_magic_method(allow_unsafe_getattr):
             raise AttributeError(name)
 
         def __dir__(self):
-            if is_py3:
-                names = object.__dir__(self)
-            else:
-                names = dir(object())
-            return ['foo', 'bar'] + names
+            return ['foo', 'bar'] + object.__dir__(self)
 
     itp = jedi.Interpreter("ca.", [{'ca': CompleteAttrs()}])
-    completions = itp.completions()
+    completions = itp.complete()
     names = [c.name for c in completions]
-    assert ('__dir__' in names) == is_py3
+    assert ('__dir__' in names) is True
     assert '__class__' in names
     assert 'foo' in names
     assert 'bar' in names
@@ -447,22 +415,21 @@ def test_name_not_findable():
 
     setattr(X, 'NOT_FINDABLE', X.hidden)
 
-    assert jedi.Interpreter("X.NOT_FINDA", [locals()]).completions()
+    assert jedi.Interpreter("X.NOT_FINDA", [locals()]).complete()
 
 
 def test_stubs_working():
     from multiprocessing import cpu_count
-    defs = jedi.Interpreter("cpu_count()", [locals()]).goto_definitions()
+    defs = jedi.Interpreter("cpu_count()", [locals()]).infer()
     assert [d.name for d in defs] == ['int']
 
 
 def test_sys_path_docstring():  # Was an issue in #1298
     import jedi
-    s = jedi.Interpreter("from sys import path\npath", line=2, column=4, namespaces=[locals()])
-    s.completions()[0].docstring()
+    s = jedi.Interpreter("from sys import path\npath", namespaces=[locals()])
+    s.complete(line=2, column=4)[0].docstring()
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Ignore Python 2, because EOL")
 @pytest.mark.parametrize(
     'code, completions', [
         ('x[0].uppe', ['upper']),
@@ -504,31 +471,37 @@ def test_simple_completions(code, completions):
     counter = collections.Counter(['asdf'])
     string = ''
 
-    defs = jedi.Interpreter(code, [locals()]).completions()
+    defs = jedi.Interpreter(code, [locals()]).complete()
     assert [d.name for d in defs] == completions
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Python 2 doesn't have lru_cache")
 def test__wrapped__():
     from functools import lru_cache
 
     @lru_cache(maxsize=128)
     def syslogs_to_df():
-            pass
+        pass
 
-    c, = jedi.Interpreter('syslogs_to_df', [locals()]).completions()
+    c, = jedi.Interpreter('syslogs_to_df', [locals()]).complete()
     # Apparently the function starts on the line where the decorator starts.
     assert c.line == syslogs_to_df.__wrapped__.__code__.co_firstlineno + 1
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Ignore Python 2, because EOL")
+def test_illegal_class_instance():
+    class X:
+        __class__ = 1
+    X.__name__ = 'asdf'
+    d, = jedi.Interpreter('foo', [{'foo': X()}]).infer()
+    v, = d._name.infer()
+    assert not v.is_instance()
+
+
 @pytest.mark.parametrize('module_name', ['sys', 'time', 'unittest.mock'])
 def test_core_module_completes(module_name):
     module = import_module(module_name)
-    assert jedi.Interpreter('module.', [locals()]).completions()
+    assert jedi.Interpreter('module.', [locals()]).complete()
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Ignore Python 2, because EOL")
 @pytest.mark.parametrize(
     'code, expected, index', [
         ('a(', ['a', 'b', 'c'], 0),
@@ -548,22 +521,20 @@ def test_partial_signatures(code, expected, index):
     b = functools.partial(func, 1)
     c = functools.partial(func, 1, c=2)
 
-    sig, = jedi.Interpreter(code, [locals()]).call_signatures()
+    sig, = jedi.Interpreter(code, [locals()]).get_signatures()
     assert sig.name == 'partial'
     assert [p.name for p in sig.params] == expected
     assert index == sig.index
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Ignore Python 2, because EOL")
 def test_type_var():
     """This was an issue before, see Github #1369"""
     import typing
     x = typing.TypeVar('myvar')
-    def_, = jedi.Interpreter('x', [locals()]).goto_definitions()
+    def_, = jedi.Interpreter('x', [locals()]).infer()
     assert def_.name == 'TypeVar'
 
 
-@pytest.mark.skipif(sys.version_info[0] == 2, reason="Ignore Python 2, because EOL")
 @pytest.mark.parametrize('class_is_findable', [False, True])
 def test_param_annotation_completion(class_is_findable):
     class Foo:
@@ -573,5 +544,213 @@ def test_param_annotation_completion(class_is_findable):
         Foo.__name__ = 'asdf'
 
     code = 'def CallFoo(x: Foo):\n x.ba'
-    def_, = jedi.Interpreter(code, [locals()]).completions()
+    def_, = jedi.Interpreter(code, [locals()]).complete()
     assert def_.name == 'bar'
+
+
+@pytest.mark.parametrize(
+    'code, column, expected', [
+        ('strs[', 5, ["'asdf'", "'fbar'", "'foo'", Ellipsis]),
+        ('strs[]', 5, ["'asdf'", "'fbar'", "'foo'", Ellipsis]),
+        ("strs['", 6, ["asdf'", "fbar'", "foo'"]),
+        ("strs[']", 6, ["asdf'", "fbar'", "foo'"]),
+        ('strs["]', 6, ['asdf"', 'fbar"', 'foo"']),
+
+        ('mixed[', 6, [r"'a\\sdf'", '1', '1.1', "b'foo'", Ellipsis]),
+        ('mixed[1', 7, ['', '.1']),
+        ('mixed[Non', 9, ['e']),
+
+        ('implicit[10', None, ['00']),
+
+        ('inherited["', None, ['blablu"']),
+    ]
+)
+def test_dict_completion(code, column, expected):
+    strs = {'asdf': 1, """foo""": 2, r'fbar': 3}
+    mixed = {1: 2, 1.10: 4, None: 6, r'a\sdf': 8, b'foo': 9}
+
+    class Inherited(dict):
+        pass
+    inherited = Inherited(blablu=3)
+
+    namespaces = [locals(), {'implicit': {1000: 3}}]
+    comps = jedi.Interpreter(code, namespaces).complete(column=column)
+    if Ellipsis in expected:
+        # This means that global completions are part of this, so filter all of
+        # that out.
+        comps = [c for c in comps if not c._name.is_value_name and not c.is_keyword]
+        expected = [e for e in expected if e is not Ellipsis]
+
+    assert [c.complete for c in comps] == expected
+
+
+@pytest.mark.parametrize(
+    'code, types', [
+        ('dct[1]', ['int']),
+        ('dct["asdf"]', ['float']),
+        ('dct[r"asdf"]', ['float']),
+        ('dct["a"]', ['float', 'int']),
+    ]
+)
+def test_dict_getitem(code, types):
+    dct = {1: 2, "asdf": 1.0}
+
+    comps = jedi.Interpreter(code, [locals()]).infer()
+    assert [c.name for c in comps] == types
+
+
+@pytest.mark.parametrize('class_is_findable', [False, True])
+@pytest.mark.parametrize(
+    'code, expected', [
+        ('DunderCls()[0]', 'int'),
+        ('dunder[0]', 'int'),
+        ('next(DunderCls())', 'float'),
+        ('next(dunder)', 'float'),
+        ('for x in DunderCls(): x', 'str'),
+        #('for x in dunder: x', 'str'),
+    ]
+)
+def test_dunders(class_is_findable, code, expected):
+    from typing import Iterator
+
+    class DunderCls:
+        def __getitem__(self, key) -> int:
+            pass
+
+        def __iter__(self, key) -> Iterator[str]:
+            pass
+
+        def __next__(self, key) -> float:
+            pass
+
+    if not class_is_findable:
+        DunderCls.__name__ = 'asdf'
+
+    dunder = DunderCls()
+
+    n, = jedi.Interpreter(code, [locals()]).infer()
+    assert n.name == expected
+
+
+def foo():
+    raise KeyError
+
+
+def bar():
+    return float
+
+
+@pytest.mark.parametrize(
+    'annotations, result, code', [
+        ({}, [], ''),
+        (None, [], ''),
+        ({'asdf': 'str'}, [], ''),
+
+        ({'return': 'str'}, ['str'], ''),
+        ({'return': 'None'}, ['NoneType'], ''),
+        ({'return': 'str().upper'}, [], ''),
+        ({'return': 'foo()'}, [], ''),
+        ({'return': 'bar()'}, ['float'], ''),
+
+        # typing is available via globals.
+        ({'return': 'typing.Union[str, int]'}, ['int', 'str'], ''),
+        ({'return': 'typing.Union["str", int]'},
+         ['int', 'str'] if sys.version_info >= (3, 9) else ['int'], ''),
+        ({'return': 'typing.Union["str", 1]'}, [], ''),
+        ({'return': 'typing.Optional[str]'}, ['NoneType', 'str'], ''),
+        ({'return': 'typing.Optional[str, int]'}, [], ''),  # Takes only one arg
+        ({'return': 'typing.Any'}, [], ''),
+
+        ({'return': 'typing.Tuple[int, str]'},
+         ['Tuple' if sys.version_info[:2] == (3, 6) else 'tuple'], ''),
+        ({'return': 'typing.Tuple[int, str]'}, ['int'], 'x()[0]'),
+        ({'return': 'typing.Tuple[int, str]'}, ['str'], 'x()[1]'),
+        ({'return': 'typing.Tuple[int, str]'}, [], 'x()[2]'),
+
+        ({'return': 'typing.List'}, ['list'], 'list'),
+        ({'return': 'typing.List[int]'}, ['list'], 'list'),
+        ({'return': 'typing.List[int]'}, ['int'], 'x()[0]'),
+        ({'return': 'typing.List[int, str]'}, [], 'x()[0]'),
+
+        ({'return': 'typing.Iterator[int]'}, [], 'x()[0]'),
+        ({'return': 'typing.Iterator[int]'}, ['int'], 'next(x())'),
+        ({'return': 'typing.Iterable[float]'}, ['float'], 'list(x())[0]'),
+
+        ({'return': 'decimal.Decimal'}, [], ''),
+        ({'return': 'lalalalallalaa'}, [], ''),
+        ({'return': 'lalalalallalaa.lala'}, [], ''),
+    ]
+)
+def test_string_annotation(annotations, result, code):
+    x = lambda foo: 1
+    x.__annotations__ = annotations
+    defs = jedi.Interpreter(code or 'x()', [locals()]).infer()
+    assert [d.name for d in defs] == result
+
+
+def test_name_not_inferred_properly():
+    """
+    In IPython notebook it is typical that some parts of the code that is
+    provided was already executed. In that case if something is not properly
+    inferred, it should still infer from the variables it already knows.
+    """
+    x = 1
+    d, = jedi.Interpreter('x = UNDEFINED; x', [locals()]).infer()
+    assert d.name == 'int'
+
+
+def test_variable_reuse():
+    x = 1
+    d, = jedi.Interpreter('y = x\ny', [locals()]).infer()
+    assert d.name == 'int'
+
+
+def test_negate():
+    code = "x = -y"
+    x, = jedi.Interpreter(code, [{'y': 3}]).infer(1, 0)
+    assert x.name == 'int'
+    value, = x._name.infer()
+    assert value.get_safe_value() == -3
+
+
+def test_complete_not_findable_class_source():
+    class TestClass():
+        ta=1
+        ta1=2
+
+    # Simulate the environment where the class is defined in
+    # an interactive session and therefore inspect module
+    # cannot find its source code and raises OSError (Py 3.10+) or TypeError.
+    TestClass.__module__ = "__main__"
+    # There is a pytest __main__ module we have to remove temporarily.
+    module = sys.modules.pop("__main__")
+    try:
+        interpreter = jedi.Interpreter("TestClass.", [locals()])
+        completions = interpreter.complete(column=10, line=1)
+    finally:
+        sys.modules["__main__"] = module
+
+    assert "ta" in [c.name for c in completions]
+    assert "ta1" in [c.name for c in completions]
+
+
+def test_param_infer_default():
+    abs_sig, = jedi.Interpreter('abs(', [{'abs': abs}]).get_signatures()
+    param, = abs_sig.params
+    assert param.name == 'x'
+    assert param.infer_default() == []
+
+
+@pytest.mark.parametrize(
+    'code, expected', [
+        ("random.triangular(", ['high=', 'low=', 'mode=']),
+        ("random.triangular(low=1, ", ['high=', 'mode=']),
+        ("random.triangular(high=1, ", ['low=', 'mode=']),
+        ("random.triangular(low=1, high=2, ", ['mode=']),
+        ("random.triangular(low=1, mode=2, ", ['high=']),
+    ],
+)
+def test_keyword_param_completion(code, expected):
+    import random
+    completions = jedi.Interpreter(code, [locals()]).complete()
+    assert expected == [c.name for c in completions if c.name.endswith('=')]
