@@ -4,58 +4,60 @@ Tests".
 """
 
 import os
+from pathlib import Path
 
 import pytest
-from jedi.file_io import FileIO, KnownContentFileIO
 
-from jedi._compatibility import find_module_py33, find_module
+import jedi
+from jedi.file_io import FileIO
 from jedi.inference import compiled
 from jedi.inference import imports
 from jedi.api.project import Project
 from jedi.inference.gradual.conversion import _stub_to_python_value_set
-from ..helpers import cwd_at, get_example_dir, test_dir, root_dir
+from jedi.inference.references import get_module_contexts_containing_name
+from ..helpers import get_example_dir, test_dir, test_dir_project, root_dir
+from jedi.inference.compiled.subprocess.functions import _find_module_py33, _find_module
 
 THIS_DIR = os.path.dirname(__file__)
 
 
-@pytest.mark.skipif('sys.version_info < (3,3)')
-def test_find_module_py33():
+def test_find_module_basic():
     """Needs to work like the old find_module."""
-    assert find_module_py33('_io') == (None, False)
+    assert _find_module_py33('_io') == (None, False)
     with pytest.raises(ImportError):
-        assert find_module_py33('_DOESNTEXIST_') == (None, None)
+        assert _find_module_py33('_DOESNTEXIST_') == (None, None)
 
 
 def test_find_module_package():
-    file_io, is_package = find_module('json')
-    assert file_io.path.endswith(os.path.join('json', '__init__.py'))
+    file_io, is_package = _find_module('json')
+    assert file_io.path.parts[-2:] == ('json', '__init__.py')
     assert is_package is True
 
 
 def test_find_module_not_package():
-    file_io, is_package = find_module('io')
-    assert file_io.path.endswith('io.py')
+    file_io, is_package = _find_module('io')
+    assert file_io.path.name == 'io.py'
     assert is_package is False
 
 
-pkg_zip_path = os.path.join(os.path.dirname(__file__),
-                            'zipped_imports',
-                            'pkg.zip')
+pkg_zip_path = Path(get_example_dir('zipped_imports', 'pkg.zip'))
 
 
 def test_find_module_package_zipped(Script, inference_state, environment):
-    sys_path = environment.get_sys_path() + [pkg_zip_path]
-    script = Script('import pkg; pkg.mod', sys_path=sys_path)
-    assert len(script.completions()) == 1
+    sys_path = environment.get_sys_path() + [str(pkg_zip_path)]
+
+    project = Project('.', sys_path=sys_path)
+    script = Script('import pkg; pkg.mod', project=project)
+    assert len(script.complete()) == 1
 
     file_io, is_package = inference_state.compiled_subprocess.get_module_info(
         sys_path=sys_path,
-        string=u'pkg',
-        full_name=u'pkg'
+        string='pkg',
+        full_name='pkg'
     )
     assert file_io is not None
-    assert file_io.path.endswith(os.path.join('pkg.zip', 'pkg', '__init__.py'))
-    assert file_io._zip_path.endswith('pkg.zip')
+    assert file_io.path.parts[-3:] == ('pkg.zip', 'pkg', '__init__.py')
+    assert file_io._zip_path.name == 'pkg.zip'
     assert is_package is True
 
 
@@ -85,45 +87,63 @@ def test_find_module_package_zipped(Script, inference_state, environment):
 
 )
 def test_correct_zip_package_behavior(Script, inference_state, environment, code,
-                                      file, package, path, skip_python2):
-    sys_path = environment.get_sys_path() + [pkg_zip_path]
-    pkg, = Script(code, sys_path=sys_path).goto_definitions()
+                                      file, package, path):
+    sys_path = environment.get_sys_path() + [str(pkg_zip_path)]
+    pkg, = Script(code, project=Project('.', sys_path=sys_path)).infer()
     value, = pkg._name.infer()
-    assert value.py__file__() == os.path.join(pkg_zip_path, 'pkg', file)
+    assert value.py__file__() == pkg_zip_path.joinpath('pkg', file)
     assert '.'.join(value.py__package__()) == package
     assert value.is_package() is (path is not None)
     if path is not None:
-        assert value.py__path__() == [os.path.join(pkg_zip_path, path)]
+        assert value.py__path__() == [str(pkg_zip_path.joinpath(path))]
+
+    value.string_names = None
+    assert value.py__package__() == []
+
+
+@pytest.mark.parametrize("code,names", [
+    ("from pkg.", {"module", "nested", "namespace"}),
+    ("from pkg.nested.", {"nested_module"})
+])
+def test_zip_package_import_complete(Script, environment, code, names):
+    sys_path = environment.get_sys_path() + [str(pkg_zip_path)]
+    completions = Script(code, project=Project('.', sys_path=sys_path)).complete()
+    assert names == {c.name for c in completions}
 
 
 def test_find_module_not_package_zipped(Script, inference_state, environment):
-    path = os.path.join(os.path.dirname(__file__), 'zipped_imports/not_pkg.zip')
+    path = get_example_dir('zipped_imports', 'not_pkg.zip')
     sys_path = environment.get_sys_path() + [path]
-    script = Script('import not_pkg; not_pkg.val', sys_path=sys_path)
-    assert len(script.completions()) == 1
+    script = Script('import not_pkg; not_pkg.val', project=Project('.', sys_path=sys_path))
+    assert len(script.complete()) == 1
 
     file_io, is_package = inference_state.compiled_subprocess.get_module_info(
-        sys_path=sys_path,
-        string=u'not_pkg',
-        full_name=u'not_pkg'
+        sys_path=map(str, sys_path),
+        string='not_pkg',
+        full_name='not_pkg'
     )
-    assert file_io.path.endswith(os.path.join('not_pkg.zip', 'not_pkg.py'))
+    assert file_io.path.parts[-2:] == ('not_pkg.zip', 'not_pkg.py')
     assert is_package is False
 
 
-@cwd_at('test/test_inference/not_in_sys_path/pkg')
-def test_import_not_in_sys_path(Script):
+def test_import_not_in_sys_path(Script, environment):
     """
     non-direct imports (not in sys.path)
 
     This is in the end just a fallback.
     """
-    a = Script(path='module.py', line=5).goto_definitions()
+    path = get_example_dir()
+    module_path = os.path.join(path, 'not_in_sys_path', 'pkg', 'module.py')
+    # This project tests the smart path option of Project. The sys_path is
+    # explicitly given to make sure that the path is just dumb and only
+    # includes non-folder dependencies.
+    project = Project(path, sys_path=environment.get_sys_path())
+    a = Script(path=module_path, project=project).infer(line=5)
     assert a[0].name == 'int'
 
-    a = Script(path='module.py', line=6).goto_definitions()
+    a = Script(path=module_path, project=project).infer(line=6)
     assert a[0].name == 'str'
-    a = Script(path='module.py', line=7).goto_definitions()
+    a = Script(path=module_path, project=project).infer(line=7)
     assert a[0].name == 'str'
 
 
@@ -144,20 +164,19 @@ def test_import_not_in_sys_path(Script):
 def test_flask_ext(Script, code, name):
     """flask.ext.foo is really imported from flaskext.foo or flask_foo.
     """
-    path = os.path.join(os.path.dirname(__file__), 'flask-site-packages')
-    completions = Script(code, sys_path=[path]).completions()
+    path = get_example_dir('flask-site-packages')
+    completions = Script(code, project=Project('.', sys_path=[path])).complete()
     assert name in [c.name for c in completions]
 
 
-@cwd_at('test/test_inference/')
 def test_not_importable_file(Script):
     src = 'import not_importable_file as x; x.'
-    assert not Script(src, path='example.py').completions()
+    assert not Script(src, path='example.py', project=test_dir_project).complete()
 
 
 def test_import_unique(Script):
     src = "import os; os.path"
-    defs = Script(src, path='example.py').goto_definitions()
+    defs = Script(src, path='example.py').infer()
     parent_contexts = [d._name._value for d in defs]
     assert len(parent_contexts) == len(set(parent_contexts))
 
@@ -167,10 +186,14 @@ def test_cache_works_with_sys_path_param(Script, tmpdir):
     bar_path = tmpdir.join('bar')
     foo_path.join('module.py').write('foo = 123', ensure=True)
     bar_path.join('module.py').write('bar = 123', ensure=True)
-    foo_completions = Script('import module; module.',
-                             sys_path=[foo_path.strpath]).completions()
-    bar_completions = Script('import module; module.',
-                             sys_path=[bar_path.strpath]).completions()
+    foo_completions = Script(
+        'import module; module.',
+        project=Project('.', sys_path=[foo_path.strpath]),
+    ).complete()
+    bar_completions = Script(
+        'import module; module.',
+        project=Project('.', sys_path=[bar_path.strpath]),
+    ).complete()
     assert 'foo' in [c.name for c in foo_completions]
     assert 'bar' not in [c.name for c in foo_completions]
 
@@ -181,7 +204,7 @@ def test_cache_works_with_sys_path_param(Script, tmpdir):
 def test_import_completion_docstring(Script):
     import abc
     s = Script('"""test"""\nimport ab')
-    abc_completions = [c for c in s.completions() if c.name == 'abc']
+    abc_completions = [c for c in s.complete() if c.name == 'abc']
     assert len(abc_completions) == 1
     assert abc_completions[0].docstring(fast=False) == abc.__doc__
 
@@ -191,72 +214,71 @@ def test_import_completion_docstring(Script):
 
 
 def test_goto_definition_on_import(Script):
-    assert Script("import sys_blabla", 1, 8).goto_definitions() == []
-    assert len(Script("import sys", 1, 8).goto_definitions()) == 1
+    assert Script("import sys_blabla").infer(1, 8) == []
+    assert len(Script("import sys").infer(1, 8)) == 1
 
 
-@cwd_at('jedi')
-def test_complete_on_empty_import(Script):
-    assert Script("from datetime import").completions()[0].name == 'import'
+def test_complete_on_empty_import(ScriptWithProject):
+    path = os.path.join(test_dir, 'whatever.py')
+    assert ScriptWithProject("from datetime import").complete()[0].name == 'import'
     # should just list the files in the directory
-    assert 10 < len(Script("from .", path='whatever.py').completions()) < 30
+    assert 10 < len(ScriptWithProject("from .", path=path).complete()) < 30
 
     # Global import
-    assert len(Script("from . import", 1, 5, 'whatever.py').completions()) > 30
+    assert len(ScriptWithProject("from . import", path=path).complete(1, 5)) > 30
     # relative import
-    assert 10 < len(Script("from . import", 1, 6, 'whatever.py').completions()) < 30
+    assert 10 < len(ScriptWithProject("from . import", path=path).complete(1, 6)) < 30
 
     # Global import
-    assert len(Script("from . import classes", 1, 5, 'whatever.py').completions()) > 30
+    assert len(ScriptWithProject("from . import classes", path=path).complete(1, 5)) > 30
     # relative import
-    assert 10 < len(Script("from . import classes", 1, 6, 'whatever.py').completions()) < 30
+    assert 10 < len(ScriptWithProject("from . import classes", path=path).complete(1, 6)) < 30
 
     wanted = {'ImportError', 'import', 'ImportWarning'}
-    assert {c.name for c in Script("import").completions()} == wanted
-    assert len(Script("import import", path='').completions()) > 0
+    assert {c.name for c in ScriptWithProject("import").complete()} == wanted
+    assert len(ScriptWithProject("import import", path=path).complete()) > 0
 
     # 111
-    assert Script("from datetime import").completions()[0].name == 'import'
-    assert Script("from datetime import ").completions()
+    assert ScriptWithProject("from datetime import").complete()[0].name == 'import'
+    assert ScriptWithProject("from datetime import ").complete()
 
 
 def test_imports_on_global_namespace_without_path(Script):
     """If the path is None, there shouldn't be any import problem"""
-    completions = Script("import operator").completions()
+    completions = Script("import operator").complete()
     assert [c.name for c in completions] == ['operator']
-    completions = Script("import operator", path='example.py').completions()
+    completions = Script("import operator", path='example.py').complete()
     assert [c.name for c in completions] == ['operator']
 
     # the first one has a path the second doesn't
-    completions = Script("import keyword", path='example.py').completions()
+    completions = Script("import keyword", path='example.py').complete()
     assert [c.name for c in completions] == ['keyword']
-    completions = Script("import keyword").completions()
+    completions = Script("import keyword").complete()
     assert [c.name for c in completions] == ['keyword']
 
 
 def test_named_import(Script):
     """named import - jedi-vim issue #8"""
     s = "import time as dt"
-    assert len(Script(s, 1, 15, '/').goto_definitions()) == 1
-    assert len(Script(s, 1, 10, '/').goto_definitions()) == 1
+    assert len(Script(s, path='/').infer(1, 15)) == 1
+    assert len(Script(s, path='/').infer(1, 10)) == 1
 
 
-@pytest.mark.skipif('True', reason='The nested import stuff is still very messy.')
-def test_goto_following_on_imports(Script):
+def test_nested_import(Script):
     s = "import multiprocessing.dummy; multiprocessing.dummy"
-    g = Script(s).goto_assignments()
+    g = Script(s).goto()
     assert len(g) == 1
     assert (g[0].line, g[0].column) != (0, 0)
 
 
-def test_goto_assignments(Script):
-    sys, = Script("import sys", 1, 10).goto_assignments(follow_imports=True)
+def test_goto(Script):
+    sys, = Script("import sys").goto(follow_imports=True)
     assert sys.type == 'module'
 
 
 def test_os_after_from(Script):
     def check(source, result, column=None):
-        completions = Script(source, column=column).completions()
+        completions = Script(source).complete(column=column)
         assert [c.name for c in completions] == result
 
     check('\nfrom os. ', ['path'])
@@ -270,12 +292,11 @@ def test_os_after_from(Script):
 
 def test_os_issues(Script):
     def import_names(*args, **kwargs):
-        return [d.name for d in Script(*args, **kwargs).completions()]
+        return [d.name for d in Script(*args).complete(**kwargs)]
 
     # Github issue #759
     s = 'import os, s'
     assert 'sys' in import_names(s)
-    assert 'path' not in import_names(s, column=len(s) - 1)
     assert 'os' in import_names(s, column=len(s) - 3)
 
     # Some more checks
@@ -291,7 +312,7 @@ def test_path_issues(Script):
     See pull request #684 for details.
     """
     source = '''from datetime import '''
-    assert Script(source).completions()
+    assert Script(source).complete()
 
 
 def test_compiled_import_none(monkeypatch, Script):
@@ -300,7 +321,7 @@ def test_compiled_import_none(monkeypatch, Script):
     """
     script = Script('import sys')
     monkeypatch.setattr(compiled, 'load_module', lambda *args, **kwargs: None)
-    def_, = script.goto_definitions()
+    def_, = script.infer()
     assert def_.type == 'module'
     value, = def_._name.infer()
     assert not _stub_to_python_value_set(value)
@@ -308,11 +329,17 @@ def test_compiled_import_none(monkeypatch, Script):
 
 @pytest.mark.parametrize(
     ('path', 'is_package', 'goal'), [
-        (os.path.join(THIS_DIR, 'test_docstring.py'), False, ('ok', 'lala', 'test_imports')),
-        (os.path.join(THIS_DIR, '__init__.py'), True, ('ok', 'lala', 'x', 'test_imports')),
+        # Both of these tests used to return relative paths to the module
+        # context that was initially given, but now we just work with the file
+        # system.
+        (os.path.join(THIS_DIR, 'test_docstring.py'), False,
+         ('test_inference', 'test_imports')),
+        (os.path.join(THIS_DIR, '__init__.py'), True,
+         ('test_inference', 'test_imports')),
     ]
 )
 def test_get_modules_containing_name(inference_state, path, goal, is_package):
+    inference_state.project = Project(test_dir)
     module = imports._load_python_module(
         inference_state,
         FileIO(path),
@@ -321,7 +348,7 @@ def test_get_modules_containing_name(inference_state, path, goal, is_package):
     )
     assert module
     module_context = module.as_context()
-    input_module, found_module = imports.get_module_contexts_containing_name(
+    input_module, found_module = get_module_contexts_containing_name(
         inference_state,
         [module_context],
         'string_that_only_exists_here'
@@ -330,23 +357,6 @@ def test_get_modules_containing_name(inference_state, path, goal, is_package):
     assert found_module.string_names == goal
 
 
-@pytest.mark.parametrize(
-    ('path', 'base_names', 'is_package', 'names'), [
-        ('/foo/bar.py', ('foo',), False, ('foo', 'bar')),
-        ('/foo/bar.py', ('foo', 'baz'), False, ('foo', 'baz', 'bar')),
-        ('/foo/__init__.py', ('foo',), True, ('foo',)),
-        ('/__init__.py', ('foo',), True, ('foo',)),
-        ('/foo/bar/__init__.py', ('foo',), True, ('foo',)),
-        ('/foo/bar/__init__.py', ('foo', 'bar'), True, ('foo', 'bar')),
-    ]
-)
-def test_load_module_from_path(inference_state, path, base_names, is_package, names):
-    file_io = KnownContentFileIO(path, '')
-    m = imports._load_module_from_path(inference_state, file_io, base_names)
-    assert m.is_package() == is_package
-    assert m.string_names == names
-
-
 @pytest.mark.parametrize(
     'path', ('api/whatever/test_this.py', 'api/whatever/file'))
 @pytest.mark.parametrize('empty_sys_path', (False, True))
@@ -359,9 +369,9 @@ def test_relative_imports_with_multiple_similar_directories(Script, path, empty_
     script = Script(
         "from . ",
         path=os.path.join(dir, path),
-        _project=project,
+        project=project,
     )
-    name, import_ = script.completions()
+    name, import_ = script.complete()
     assert import_.name == 'import'
     assert name.name == 'api_test1'
 
@@ -372,39 +382,41 @@ def test_relative_imports_with_outside_paths(Script):
     script = Script(
         "from ...",
         path=os.path.join(dir, 'api/whatever/test_this.py'),
-        _project=project,
+        project=project,
     )
-    assert [c.name for c in script.completions()] == ['api', 'import', 'whatever']
+    assert [c.name for c in script.complete()] == ['api', 'whatever']
 
     script = Script(
         "from " + '.' * 100,
         path=os.path.join(dir, 'api/whatever/test_this.py'),
-        _project=project,
+        project=project,
     )
-    assert [c.name for c in script.completions()] == ['import']
+    assert not script.complete()
 
 
-@cwd_at('test/examples/issue1209/api/whatever/')
 def test_relative_imports_without_path(Script):
-    project = Project('.', sys_path=[], smart_sys_path=False)
-    script = Script("from . ", _project=project)
-    assert [c.name for c in script.completions()] == ['api_test1', 'import']
+    path = get_example_dir('issue1209', 'api', 'whatever')
+    project = Project(path, sys_path=[], smart_sys_path=False)
+    script = Script("from . ", project=project)
+    assert [c.name for c in script.complete()] == ['api_test1', 'import']
 
-    script = Script("from .. ", _project=project)
-    assert [c.name for c in script.completions()] == ['import', 'whatever']
+    script = Script("from .. ", project=project)
+    assert [c.name for c in script.complete()] == ['import', 'whatever']
 
-    script = Script("from ... ", _project=project)
-    assert [c.name for c in script.completions()] == ['api', 'import', 'whatever']
+    script = Script("from ... ", project=project)
+    assert [c.name for c in script.complete()] == ['api', 'import', 'whatever']
 
 
 def test_relative_import_out_of_file_system(Script):
-    script = Script("from " + '.' * 100)
-    import_, = script.completions()
+    code = "from " + '.' * 100
+    assert not Script(code).complete()
+    script = Script(code + ' ')
+    import_, = script.complete()
     assert import_.name == 'import'
 
     script = Script("from " + '.' * 100 + 'abc import ABCMeta')
-    assert not script.goto_definitions()
-    assert not script.completions()
+    assert not script.infer()
+    assert not script.complete()
 
 
 @pytest.mark.parametrize(
@@ -433,48 +445,66 @@ def test_import_name_calculation(Script):
 
 @pytest.mark.parametrize('name', ('builtins', 'typing'))
 def test_pre_defined_imports_module(Script, environment, name):
-    if environment.version_info.major < 3 and name == 'builtins':
-        name = '__builtin__'
-
     path = os.path.join(root_dir, name + '.py')
     module = Script('', path=path)._get_module_context()
     assert module.string_names == (name,)
 
-    assert module.inference_state.builtins_module.py__file__() != path
-    assert module.inference_state.typing_module.py__file__() != path
+    assert str(module.inference_state.builtins_module.py__file__()) != path
+    assert str(module.inference_state.typing_module.py__file__()) != path
 
 
 @pytest.mark.parametrize('name', ('builtins', 'typing'))
 def test_import_needed_modules_by_jedi(Script, environment, tmpdir, name):
-    if environment.version_info.major < 3 and name == 'builtins':
-        name = '__builtin__'
-
     module_path = tmpdir.join(name + '.py')
     module_path.write('int = ...')
     script = Script(
         'import ' + name,
         path=tmpdir.join('something.py').strpath,
-        sys_path=[tmpdir.strpath] + environment.get_sys_path(),
+        project=Project('.', sys_path=[tmpdir.strpath] + environment.get_sys_path()),
     )
-    module, = script.goto_definitions()
-    assert module._inference_state.builtins_module.py__file__() != module_path
-    assert module._inference_state.typing_module.py__file__() != module_path
+    module, = script.infer()
+    assert str(module._inference_state.builtins_module.py__file__()) != module_path
+    assert str(module._inference_state.typing_module.py__file__()) != module_path
 
 
 def test_import_with_semicolon(Script):
-    names = [c.name for c in Script('xzy; from abc import ').completions()]
+    names = [c.name for c in Script('xzy; from abc import ').complete()]
     assert 'ABCMeta' in names
     assert 'abc' not in names
 
 
 def test_relative_import_star(Script):
     # Coming from github #1235
-    import jedi
-
     source = """
     from . import *
     furl.c
     """
-    script = jedi.Script(source, 3, len("furl.c"), 'export.py')
+    script = Script(source, path='export.py')
 
-    assert script.completions()
+    assert script.complete(3, len("furl.c"))
+
+
+@pytest.mark.parametrize('with_init', [False, True])
+def test_relative_imports_without_path_and_setup_py(
+        Script, inference_state, environment, tmpdir, with_init):
+    # Contrary to other tests here we create a temporary folder that is not
+    # part of a folder with a setup.py that signifies
+    tmpdir.join('file1.py').write('do_foo = 1')
+    other_path = tmpdir.join('other_files')
+    other_path.join('file2.py').write('def do_nothing():\n pass', ensure=True)
+    if with_init:
+        other_path.join('__init__.py').write('')
+
+    for name, code in [('file2', 'from . import file2'),
+                       ('file1', 'from .. import file1')]:
+        for func in (jedi.Script.goto, jedi.Script.infer):
+            n, = func(Script(code, path=other_path.join('test1.py').strpath))
+            assert n.name == name
+            assert n.type == 'module'
+            assert n.line == 1
+
+
+def test_import_recursion(Script):
+    path = get_example_dir('import-recursion', "cq_example.py")
+    for c in Script(path=path).complete(3, 3):
+        c.docstring()
