@@ -6,7 +6,7 @@ def test_error_correction_with(Script):
     with open() as f:
         try:
             f."""
-    comps = Script(source).completions()
+    comps = Script(source).complete()
     assert len(comps) > 30
     # `open` completions have a closed attribute.
     assert [1 for c in comps if c.name == 'closed']
@@ -23,14 +23,14 @@ def test_string_literals(Script):
 
     script = Script(dedent(source))
     assert script._get_module_context().tree_node.end_pos == (6, 0)
-    assert script.completions()
+    assert not script.complete()
 
 
 def test_incomplete_function(Script):
     source = '''return ImportErr'''
 
-    script = Script(dedent(source), 1, 3)
-    assert script.completions()
+    script = Script(dedent(source))
+    assert script.complete(1, 3)
 
 
 def test_decorator_string_issue(Script):
@@ -46,5 +46,5 @@ def test_decorator_string_issue(Script):
     bla.''')
 
     s = Script(source)
-    assert s.completions()
+    assert s.complete()
     assert s._get_module_context().tree_node.get_code() == source
